 /**
 * \brief Mandelbrot kernel function.
 * \details Evaluates whether a set of four adjacent pixels are part of the Mandelbrot
 *          set. The output is an array where each value represents how many iterations
 *          were required to be able to decide whether the point in imaginary space
 *          is part of the Mandelbrot set or not. A value of MAX_ITER represents a point
 *          that is part of the Mandelbrot set. All other values represent points that are
 *          not part of the Mandelbrot set.
 *          A value is in the Mandelbrot set if the results of the calculations remain bounded.
 * \param[out] output Output data buffer. Must be width * height * sizeof(cl_uchar) in size.
 * \param[in] width Width of the data required.
 * \param[in] height Height of the data required.
 */
__kernel void rgb2yuv(__global uchar *py,
												 __global uchar *pu,
												 __global uchar *pv,
												 __global uchar* pr,	
												 __global uchar* pg,	
												 __global uchar* pb,	
                         const int width,
                         const int height)
{
    /* [Kernel size] */
    /*
     * Each kernel calculates 4 output pixels in the same row (hence the '* 4').
     * x is in the range [0, width] .
     * y is in the range [0, height].
     */
    int x = get_global_id(0);
    int y = get_global_id(1);

    /* [Kernel size] */
		uchar u,v,r,g,b,a;
		int pos = width*y + x;
		int rgbpos = pos*4;

		#if 0
		r = pr[pos];
		g = pg[pos];
		b = pb[pos];
		#else
		r = pr[rgbpos];
		g = pr[rgbpos+1];
		b = pr[rgbpos+2];
		a = pr[rgbpos+3];
		#endif

    py[width*y + x] = (uchar)( ( 66 * r + 129 * g + 25 * b + 128) >> 8) + 16 ;
    u = (uchar)( ( -38 * r - 74 * g + 112 * b + 128) >> 8) + 128 ;
    v = (uchar)( ( 112 * r - 94 * g - 18 * b + 128) >> 8) + 128 ;	
		if (pu == pv) {
			//NV12 //or NV21	
      if (y%2 == 0){ 
          if (x%2 == 0) { 
          		pu[width * (y>>1) + x] = u;
          } else {
              pv[width * (y>>1) + x] = v;
          }
      }				
			
		} else {
			//I420 or YV12	
      if (y%2 == 0){ 
          if (x%2 == 0) { 
          		pu[(width>>1) * (y>>1) + (x>>1)] = u;
          } else {
              pv[(width>>1) * (y>>1) + (x>>1)] = v;
          }
      }			
		}
}
